# Copyright (c) 2000-2008, JPackage Project
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the JPackage Project nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

%define with()          %{expand:%%{?with_%{1}:1}%%{!?with_%{1}:0}}
%define without()       %{expand:%%{?with_%{1}:0}%%{!?with_%{1}:1}}
%define bcond_with()    %{expand:%%{?_with_%{1}:%%global with_%{1} 1}}
%define bcond_without() %{expand:%%{!?_without_%{1}:%%global with_%{1} 1}}

%bcond_with bootstrap

# If you don't want to run the unit tests
# give rpmbuild option '--without tests'
# (dwalluck) This option is required for bootstrapping
%if %with bootstrap
%bcond_with tests
%else
%bcond_without tests
%endif

%define _with_repolib 1

# If you want repolib package to be built,
# issue the following: 'rpmbuild --with repolib'
%define with_repolib %{?_with_repolib:1}%{!?_with_repolib:0}
%define without_repolib %{!?_with_repolib:1}%{?_with_repolib:0}

%define repodir %{_javadir}/repository.jboss.com/jboss/serialization/%{version}.GA-brew
%define repodirlib %{repodir}/lib
%define repodirsrc %{repodir}/src

%define section		free

Name:		jboss-serialization
Version:	1.0.3
Release:	5%{?dist}
Epoch:		0
Summary:	JBoss Serialization
License:	LGPLv2+
Group:		Development/Java
URL:		http://labs.jboss.com/portal/index.html?ctrl:id=page.default.info&project=serialization
# cvs -d:pserver:anonymous@anoncvs.forge.jboss.com:/cvsroot/jboss export -r JBSER_1_0_3_GA jboss-serialization
# mv jboss-serialization jboss-serialization-1.0.3.GA-src
# tar czf jboss-serialization-1.0.3.GA-src.tar.gz jboss-serialization-1.0.3.GA-src
Source0:	jboss-serialization-1.0.3.GA-src.tar.gz
Source1:	jboss-serialization-component-info.xml
Patch0:		jboss-serialization-build_xml.patch
Patch1:		jboss-serialization-ProxySerializationTestCase.patch
Patch2:		jboss-serialization-MockHibernateSession.patch
BuildArch:	noarch
BuildRequires:	jpackage-utils >= 0:1.6
BuildRequires:	java-devel >= 0:1.5
BuildRequires:	ant >= 0:1.6
BuildRequires:	asm
BuildRequires:	cglib
BuildRequires:	concurrent
BuildRequires:	jakarta-commons-logging
BuildRequires:	javassist
BuildRequires:	jboss-common
BuildRequires:	jboss-aop
BuildRequires:	jboss-jrunit
BuildRequires:	jboss-profiler
%if %with tests
BuildRequires:	ant-junit
BuildRequires:	jbossas
BuildRequires:	jboss-remoting
%endif
BuildRequires:	jcommon
BuildRequires:	jfreechart
BuildRequires:	jgroups >= 0:2.2.9.2
BuildRequires:	joda-time
BuildRequires:	junit
BuildRequires:	log4j
BuildRequires:	gnu-trove
BuildRequires:	hibernate3

Requires:	jpackage-utils >= 0:1.6
Requires:	log4j
Requires:	gnu-trove
BuildRoot:	%{_tmppath}/%{name}-%{version}-%{release}-root

%description
We (java developers) have accepted over the years 
java.io.ObjectInputStream and java.io.ObjectOutputStream 
being slow when dealing with writeObject operations.
We then started using Externalizable objects as a faster 
approach for serialization, but even that way was slow when 
using writeObject operations inside externalizable classes.
Recently we discovered that most of the problems in 
JavaSerialization are related to static synchronized caching, 
what causes CPU spikes and also diminishes scaling capabilities.
With JBossSerialization we have done internal benchmarks and we 
have realized at least 2 times faster serialization with this 
library. These benchmarks are commited into our CVS repository 
(as testcases) and they are publicly available.
The main feature in JBossSerialization besides performance, is 
Smart Cloning. Smart cloning is the capability of the reuse of 
final fields among different class loaders doing exactly what 
serialization does, without the need of convert every field into 
a byteArray. This approach is at least 10 times faster than using 
serialization over a byte array.

%if %{with_repolib}
%package repolib
Summary:        Artifacts to be uploaded to a repository library
Group:	        Development/Libraries/Java

%description repolib
Artifacts to be uploaded to a repository library.
This package is not meant to be installed but so its contents
can be extracted through rpm2cpio.
%endif

%package javadoc
Summary:        Javadoc for %{name}
Group:          Development/Documentation

%description javadoc
%{summary}.

%prep
%setup -q -n %{name}-%{version}.GA-src
chmod -R go=u-w *
find . -name '*.jar' | xargs -t %{__rm}
%patch0 -p0
# Patch1 is needed if newer Hibernate versions are used
%patch1 -p0
%patch2 -p0

pushd libs
#BUILD/jboss-serialization/libs/asm-attrs.jar.no
ln -sf $(build-classpath asm/asm-attrs) .
#BUILD/jboss-serialization/libs/asm.jar.no
ln -sf $(build-classpath asm/asm) .
#BUILD/jboss-serialization/libs/cglib-2.1_2jboss.jar.no
ln -sf $(build-classpath cglib) cglib-2.1_2jboss.jar
#BUILD/jboss-serialization/libs/commons-logging.jar.no
ln -sf $(build-classpath commons-logging) .
#BUILD/jboss-serialization/libs/concurrent.jar.no
ln -sf $(build-classpath concurrent) .
%if %with tests
#BUILD/jboss-serialization/libs/hibernate3.jar.no
# FIXME a jboss-serialization test is using a class 
# org.hibernate.engine.query.NativeSQLQuerySpecification that has been
# removed from Hibernate 3.2.4 (existed in 3.2.0)
ln -sf $(build-classpath hibernate3) .
# (dwalluck): removing the test case
%{__rm} ../tests/org/jboss/serial/hibernatesession/MockHibernateSession.java
%{__rm} ../tests/org/jboss/serial/hibernatesession/ProxySerializationTestCase.java
#mv hibernate3.jar.no hibernate3.jar
%endif
#BUILD/jboss-serialization/libs/javassist/lib/javassist.jar.no
pushd javassist/lib
ln -sf $(build-classpath javassist) .
popd
#BUILD/jboss-serialization/libs/jboss/aop/lib/common-softvaluehashmap.jar.no
#BUILD/jboss-serialization/libs/jboss/aop/lib/jboss-aop.jar.no
#BUILD/jboss-serialization/libs/jboss/aop/lib/jboss-aop-jdk50-client.jar.no
#BUILD/jboss-serialization/libs/jboss/aop/lib/jboss-aop-jdk50.jar.no
#BUILD/jboss-serialization/libs/jboss/aop/lib/jdk14-pluggable-instrumentor.jar.no
#BUILD/jboss-serialization/libs/jboss/aop/lib/jrockit-pluggable-instrumentor.jar.no
#BUILD/jboss-serialization/libs/jboss/aop/lib/pluggable-instrumentor.jar.no
pushd jboss/aop/lib
ln -sf $(build-classpath jboss-aop/jboss-aop) .
ln -sf $(build-classpath jboss-aop/jboss-aop-jdk50-client) .
ln -sf $(build-classpath jboss-aop/jboss-aop-jdk50) .
ln -sf $(build-classpath jboss-aop/jdk14-pluggable-instrumentor) .
ln -sf $(build-classpath jboss-aop/jrockit-pluggable-instrumentor) .
ln -sf $(build-classpath jboss-aop/pluggable-instrumentor) .
popd
#BUILD/jboss-serialization/libs/jboss-common.jar.no
ln -sf $(build-classpath jboss-common/jboss-common) .
#BUILD/jboss-serialization/libs/jboss-profiler-jvmti.jar.no
ln -sf $(build-classpath jboss-profiler/jboss-profiler-jvmti) .
%if %with tests
#BUILD/jboss-serialization/libs/jboss-client.jar.no
ln -sf $(build-classpath jbossas/jboss-client) .
#BUILD/jboss-serialization/libs/jboss-j2ee.jar.no
ln -sf $(build-classpath jbossas/jboss-j2ee) .
#BUILD/jboss-serialization/libs/jboss-remoting.jar.no
ln -sf $(build-classpath jboss-remoting) .
%endif
#BUILD/jboss-serialization/libs/jcommon-1.0.0-rc1.jar.no
ln -sf $(build-classpath jcommon) .
#BUILD/jboss-serialization/libs/jfreechart-1.0.0-rc1.jar.no
ln -sf $(build-classpath jfreechart) .
#BUILD/jboss-serialization/libs/jgroups.jar.no
ln -sf $(build-classpath jgroups) .
#BUILD/jboss-serialization/libs/joda-time-1.2.1.jar.no
ln -sf $(build-classpath joda-time) joda-time-1.2.1.jar
#BUILD/jboss-serialization/libs/jrunit.jar.no
ln -sf $(build-classpath jboss-jrunit) jrunit.jar
#BUILD/jboss-serialization/libs/junit.jar.no
ln -sf $(build-classpath junit) .
#BUILD/jboss-serialization/libs/log4j.jar.no
ln -sf $(build-classpath log4j) .
#BUILD/jboss-serialization/libs/profilerConsole.jar.no
ln -sf $(build-classpath jboss-profiler/profilerConsole) .
#BUILD/jboss-serialization/libs/trove.jar.no
ln -sf $(build-classpath gnu-trove) trove.jar
popd

%build
export OPT_JAR_LIST="ant/ant-junit junit"
export CLASSPATH=
%if %with tests
export ANT_OPTS="-Djava.net.preferIPv4Stack=true"
ant jar javadocs run-tests 
%else
ant jar javadocs
%endif

%install
rm -rf $RPM_BUILD_ROOT

# jar
install -d -m 0755 $RPM_BUILD_ROOT%{_javadir}
install -p -m 0644 build/jar/%{name}.jar $RPM_BUILD_ROOT%{_javadir}/%{name}-%{version}.jar
(cd $RPM_BUILD_ROOT%{_javadir} && for jar in *-%{version}*; do ln -sf ${jar} `echo $jar| sed  "s|-%{version}||g"`; done)

# javadoc
install -d -m 755 $RPM_BUILD_ROOT%{_javadocdir}/%{name}-%{version}
cp -pr build/api/* $RPM_BUILD_ROOT%{_javadocdir}/%{name}-%{version}
ln -s %{name}-%{version} $RPM_BUILD_ROOT%{_javadocdir}/%{name} # ghost symlink

%if %{with_repolib}
	install -d -m 755 $RPM_BUILD_ROOT%{repodir}
	install -d -m 755 $RPM_BUILD_ROOT%{repodirlib}
	install -p -m 644 %{SOURCE1} $RPM_BUILD_ROOT%{repodir}/component-info.xml
        tag=`echo %{name}-%{version}-%{release} | sed 's|\.|_|g'`
        sed -i "s/@TAG@/$tag/g" $RPM_BUILD_ROOT%{repodir}/component-info.xml
        sed -i "s/@VERSION@/%{version}.GA-brew/g" $RPM_BUILD_ROOT%{repodir}/component-info.xml
	install -d -m 755 $RPM_BUILD_ROOT%{repodirsrc}
	install -p -m 644 %{PATCH0} $RPM_BUILD_ROOT%{repodirsrc}
	install -p -m 644 %{PATCH2} $RPM_BUILD_ROOT%{repodirsrc}
	install -p -m 644 %{SOURCE0} $RPM_BUILD_ROOT%{repodirsrc}
	cp -p $RPM_BUILD_ROOT%{_javadir}/jboss-serialization.jar $RPM_BUILD_ROOT%{repodirlib}
%endif

%clean
rm -rf $RPM_BUILD_ROOT

%files
%defattr(0644,root,root,0755)
%doc docs/readme.txt
%{_javadir}/%{name}-%{version}.jar
%{_javadir}/%{name}.jar

%files javadoc
%defattr(0644,root,root,0755)
%{_javadocdir}/%{name}-%{version}
%{_javadocdir}/%{name}

%if %{with_repolib}
%files repolib
%defattr(0644,root,root,0755)
%{_javadir}/repository.jboss.com
%endif

%changelog
* Mon Dec 01 2008 David Walluck <dwalluck@redhat.com> 0:1.0.3-5
- rebuild

* Mon Dec 01 2008 David Walluck <dwalluck@redhat.com> 0:1.0.3-4
- use explicit file list
- fix component-info.xml
- fix repolib file ownership
- fix repolib permissions
- rediff patches
- move symlinking to %%prep

* Sun Jun 22 2008 David Walluck <dwalluck@redhat.com> 0:1.0.3-3.jpp5
- build as non-bootstrap

* Mon Jun 02 2008 David Walluck <dwalluck@redhat.com> 0:1.0.3-2.jpp5
- remove javadoc scriptlets
- fix %%doc
- fix permissions

* Mon Jun 25 2007 Fernando Nasser <fnasser@redhat.com> 0:1.0.3-1jpp.ep.3
- Rebuild with fixed jboss-profiler package, now containing profilerConsole

* Fri Jun 22 2007 Fernando Nasser <fnasser@redhat.com> 0:1.0.3-1jpp.ep.2
- Rebuild

* Fri Jun 22 2007 Fernando Nasser <fnasser@redhat.com> 0:1.0.3-1jpp.ep.1
- Merge with upstream for upgrade
- Add repolib subpackage

* Fri Jun 22 2007 Fernando Nasser <fnasser@redhat.com> 0:1.0.3-1jpp
- Upgrade to 1.0.3.GA

* Fri Jun 22 2007 Fernando Nasser <fnasser@redhat.com> - 0:1.0.0-4jpp.ep.1
- Use jboss-common

* Mon Jul 31 2006 Fernando Nasser <fnasser@redhat.com> - 0:1.0.0-4jpp_0rh
- Merge with upstream

* Thu Jul 13 2006 Ralph Apel <r.apel at r-apel.de> 0:1.0.0-4jpp
- Conditionalize libs for build with tests

* Wed Jul 12 2006 Fernando Nasser <fnasser@redhat.com> - 0:1.0.0-3jpp_0rh
- Merge with upstream
- Bootstraping (without tests)

* Wed Jul 12 2006 Ralph Apel <r.apel at r-apel.de> 0:1.0.0-3jpp
- Move jboss4-j2ee BR for build with tests

* Mon Jun 26 2006 Ralph Apel <r.apel at r-apel.de> 0:1.0.0-2jpp
- Fix (Build)Requires:

* Tue Jun 09 2006 Ralph Apel <r.apel at r-apel.de> 0:1.0.1-1jpp
- Upgrade to 1.0.0.GA

* Wed Jun 07 2006 Fernando Nasser <fnasser@redhat.com> 0:1.0.0-0.rc2.2jpp_1rh
- Merge with upstream

* Wed May 31 2006 Ralph Apel <r.apel at r-apel.de> 0:1.0.0-0.rc2.2jpp
- add -Djava.net.preferIPv4Stack=true on each JVM

* Fri May 26 2006 Fernando Nasser <fnasser@redhat.com> 0:1.0.0-0.rc2.1jpp_1rh
- First Red Hat build
- Temporarely disable tests as they hang

* Wed May 03 2006 Ralph Apel <r.apel at r-apel.de> 0:1.0.0-0.rc2.1jpp
- First JPP-1.7 release
